# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sr', 'sr.robot3']

package_data = \
{'': ['*']}

install_requires = \
['astoria>=0.4.0,<0.5.0', 'j5>=0.13.0,<0.14.0', 'pyserial>=3.4,<4.0']

setup_kwargs = {
    'name': 'sr.robot3',
    'version': '2022.0.0a1',
    'description': 'Student Robotics API for Python 3',
    'long_description': '# sr.robot3\n\n![Tests](https://github.com/srobo/sr-robot3/workflows/Tests/badge.svg)\n[![PyPI version](https://badge.fury.io/py/sr.robot3.svg)](https://pypi.org/project/sr.robot3/)\n[![MIT license](https://img.shields.io/badge/license-MIT-brightgreen.svg?style=flat)](https://opensource.org/licenses/MIT)\n![Bees](https://img.shields.io/badge/bees-110%25-yellow.svg)\n\nStudent Robotics API for Python 3\n\nThis package uses [PEP420]()-style namespace packaging.\n\n## Development\n\n### Requirements\n\nThis project uses the [Poetry](https://python-poetry.org) dependency and virtualenv manager.\n\nYou will also need:\n\n- Python 3.6+\n- Make\n\n### Setup\n\n- Clone the repository from GitHub to a folder on your local machine\n- `cd` to that folder, and tell Poetry to install dependencies and set up a virtualenv `poetry install`\n- You can now enter the virtual environment using `poetry shell` and develop using your IDE of choice.\n\n### Tests\n\nThe full type, test and lint suite can be run using make: `make`.\n\nYou can also run parts of the suite.\n\n- Unit tests: `make test`\n- Unit tests with HTML coverage: `make test-cov`\n- Linting: `make lint`\n- Static type checks: `make type`\n\n## Contributions\n\nThis project is released under the MIT Licence. For more information, please see LICENSE.\n\nThe CONTRIBUTORS file can be generated by executing CONTRIBUTORS.gen. This generated file contains a list of people who have contributed to sr.robot3.\n',
    'author': 'Student Robotics',
    'author_email': 'kit-brain@studentrobotics.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/srobo/sr-robot3',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
